<?php

namespace Drupal\transmuter\Attribute;

use Drupal\transmuter\Plugin\TransmuterDefinition;
use Drupal\themespace\Attribute\ProviderTypedPlugin;

/**
 * Definition attribute for transmuter plugins.
 */
#[\Attribute(\Attribute::TARGET_CLASS)]
class Transmuter extends ProviderTypedPlugin {

  /**
   * Constructs a new instance of the Transmuter plugin attribute.
   *
   * @param string $id
   *   The transmuter plugin identifier.
   * @param string|string[] $hook
   *   Either a single preprocess hook name or an array of preprocess hooks.
   * @param int $priority
   *   The hook priority of the transmuter. Smaller numbers will execute the
   *   same hook sooner than priorities that are larger numbers. Normal hook
   *   specificity rules still supercede this priority. It only applies to
   *   resolving the order of the same hook.
   * @param class-string|null $deriver
   *   The plugin deriver for deriving transmuter plugin definitions.
   */
  public function __construct(
    public readonly string $id,
    public readonly string|array $hook,
    public readonly ?int $priority = 0,
    public readonly ?string $deriver = NULL,
  ) {}

  /**
   * {@inheritdoc}
   *
   * @return \Drupal\transmuter\Plugin\TransmuterDefinition
   *   The transmuter plugin definition instance.
   */
  public function get(): TransmuterDefinition {
    return new TransmuterDefinition(parent::get());
  }

}
